<?php
/**
 * WPSEO plugin file.
 *
 * @package WPSEO\CLI
 */

use WP_CLI\Dispatcher\CommandNamespace;

/**
 * Lists, creates, updates, deletes and follows Yoast SEO redirects.
 */
final class WPSEO_CLI_Redirect_Command_Namespace extends CommandNamespace {
	// Intentionally left empty.
}
